@@ -19,6 +19,7 @@
 from Base import BaseRepository
 from offlineimap import folder, imaputil
 import re
+from threading import *
 
 class IMAPRepository(BaseRepository):
     def __init__(self, config, accountname, imapserver):
@@ -27,29 +28,41 @@ class IMAPRepository(BaseRepository):
         self.imapserver = imapserver
         self.config = config
         self.accountname = accountname
-        self.imapobj = imapserver.makeconnection()
         self.folders = None
         self.nametrans = lambda foldername: foldername
+        self.folderfilter = lambda foldername: 1
         if config.has_option(accountname, 'nametrans'):
             self.nametrans = eval(config.get(accountname, 'nametrans'))
+        if config.has_option(accountname, 'folderfilter'):
+            self.folderfilter = eval(config.get(accountname, 'folderfilter'))
 
     def getsep(self):
         return self.imapserver.delim
 
     def getfolder(self, foldername):
         return folder.IMAP.IMAPFolder(self.imapserver, foldername,
-                                      self.nametrans(foldername))
+                                      self.nametrans(foldername),
+                                      accountname)
 
     def getfolders(self):
         if self.folders != None:
             return self.folders
         retval = []
-        for string in self.imapobj.list()[1]:
+        imapobj = self.imapserver.acquireconnection()
+        try:
+            listresult = imapobj.list()[1]
+        finally:
+            self.imapserver.releaseconnection(imapobj)
+        for string in listresult:
             flags, delim, name = imaputil.imapsplit(string)
             if '\\Noselect' in imaputil.flagsplit(flags):
                 continue
+            foldername = imaputil.dequote(name)
+            if not self.folderfilter(foldername):
+                continue
             retval.append(folder.IMAP.IMAPFolder(self.imapserver, name,
-                                                 self.nametrans(imaputil.dequote(name))))
+                                                 self.nametrans(foldername),
+                                                 self.accountname))
         retval.sort(lambda x, y: cmp(x.getvisiblename(), y.getvisiblename()))
         self.folders = retval
         return retval
